/* -*- mode: c++; c-basic-offset: 4; indent-tabs-mode: nil -*-
   this file is part of rcssserver3D
   Fri May 9 2003
   Copyright (C) 2003 Koblenz University
   $Id: spark.cpp,v 1.3 2004/04/25 17:04:23 rollmark Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "spark.h"
#include <zeitgeist/zeitgeist.h>

//#include <SDL/SDL.h>
using namespace spark;
using namespace kerosin;
using namespace oxygen;
using namespace zeitgeist;
using namespace salt;
using namespace std;
using namespace boost;


Spark::Spark(const string& relPathPrefix) :
    mZeitgeist("." PACKAGE_NAME, relPathPrefix),
    mOxygen(mZeitgeist),
    mKerosin(mZeitgeist)
{
}

Spark::~Spark()
{
    // reset shared ptr to objects in the zeitgeist hierarchy before
    // the zeitgeist core is shutdown
    mLogServer.reset();
    mScriptServer.reset();
    mSceneServer.reset();
    mFPSController.reset();
    mSimulationServer.reset();
}

bool Spark::Init(int argc, char** argv)
{
    mLogServer = mZeitgeist.GetCore()->GetLogServer();
    if (mLogServer.get() == 0)
        {
            cout << "(Spark) ERROR: LogServer not found\n";
            return false;
        }

    mScriptServer = mZeitgeist.GetCore()->GetScriptServer();
    if (mScriptServer.get() == 0)
        {
            mLogServer->Error()
                << "(Spark) ERROR: ScriptServer not found\n";
            return false;
        }

    // run the spark init script
    mZeitgeist.GetCore()->GetRoot()->GetScript()->RunInitScript
        (
         "spark.rb",
         "lib/spark",
         ScriptServer::IS_COMMON
         );

    mSceneServer = shared_dynamic_cast<SceneServer>
        (mZeitgeist.GetCore()->Get("/sys/server/scene"));

    if (mSceneServer.get() == 0)
        {
            mLogServer->Error() << "(Spark) ERROR: SceneServer not found\n";
            return false;
        }

    mSimulationServer = shared_dynamic_cast<SimulationServer>
        (mZeitgeist.GetCore()->Get("/sys/server/simulation"));

    if (mSimulationServer.get() == 0)
        {
            mLogServer->Error() << "(Spark) ERROR: SimulationServer not found\n";
            return false;
        }

    // run the app defined init
    return InitApp(argc,argv);
}

bool Spark::InitApp(int /*argc*/, char** /*argv*/)
{
    return true;
}

Zeitgeist& Spark::GetZeitgeist()
{
    return mZeitgeist;
}

shared_ptr<Core> Spark::GetCore()
{
    return mZeitgeist.GetCore();
}

shared_ptr<zeitgeist::LogServer> Spark::GetLog()
{
    return mZeitgeist.GetCore()->GetLogServer();
}

shared_ptr<SceneServer> Spark::GetSceneServer()
{
    return mSceneServer;
}

shared_ptr<SimulationServer> Spark::GetSimulationServer()
{
    return mSimulationServer;
}

shared_ptr<InputControl> Spark::GetInputControl()
{
    if (mSimulationServer.get() == 0)
        {
            return shared_ptr<InputControl>();
        }

    return shared_dynamic_cast<InputControl>
        (mSimulationServer->GetControlNode("kerosin/InputControl"));
}

shared_ptr<RenderControl> Spark::GetRenderControl()
{
    if (mSimulationServer.get() == 0)
        {
            return shared_ptr<RenderControl>();
        }

    return shared_dynamic_cast<RenderControl>
        (mSimulationServer->GetControlNode("kerosin/RenderControl"));
}


shared_ptr<ScriptServer> Spark::GetScriptServer()
{
    return mScriptServer;
}

shared_ptr<Scene> Spark::GetActiveScene()
{
    shared_ptr<Scene> scene = mSceneServer->GetActiveScene();

    if (scene.get() == 0)
        {
            mLogServer->Warning()
                << "(Spark) Warning: no active scene registered\n";
        }

    return scene;
}

