/* -*- mode: c++; c-basic-offset: 4; indent-tabs-mode: nil -*-

   this file is part of rcssserver3D
   Fri May 9 2003
   Copyright (C) 2002,2003 Koblenz University
   Copyright (C) 2003 RoboCup Soccer Server 3D Maintenance Group
   $Id: tcpsocket.cpp,v 1.2 2004/04/25 19:24:04 rollmark Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include "tcpsocket.hpp"
#include "exception.hpp"
#include <sys/types.h>
#include <sys/socket.h>
#include <cerrno>

namespace rcss
{
    namespace net
    {
        TCPSocket::TCPSocket()
        {
            open();
        }

        TCPSocket::TCPSocket( const Addr& addr )
        {
            open();
            bind( addr );
        }

        TCPSocket::TCPSocket( const Addr& addr, const Addr& dest )
        {
            open();
            bind( addr );
            connect( dest );
        }

        TCPSocket::TCPSocket( int socket )
        {
            m_open      = true;
            m_connected = true;
            m_socket    = socket;
        }

        void
        TCPSocket::doOpen( int& fd )
        {
            close();
            fd = ::socket(AF_INET, SOCK_STREAM, 0 );
            if( fd < 0 )
                throw OpenErr( errno );
        }


        Socket* TCPSocket::accept(Addr& addr)
        {
            socklen_t len = sizeof(struct sockaddr);
            int fd = ::accept(
                              m_socket,
                              (struct sockaddr *)&( addr.getAddr() ),
                              &len
                              );

            if (fd < 0)
                {
                    throw AcceptErr( errno );
                }

            return new TCPSocket(fd);
        }

    }
}
